@@ -298,7 +298,7 @@ phosh_app_grid_button_class_init (PhoshAppGridButtonClass *klass)
 
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
-  gtk_widget_class_set_template_from_resource (widget_class, "/sm/puri/phosh/ui/app-grid-button.ui");
+  gtk_widget_class_set_template_from_resource (widget_class, "/mobi/phosh/ui/app-grid-button.ui");
 
   gtk_widget_class_bind_template_child_private (widget_class, PhoshAppGridButton, icon);
   gtk_widget_class_bind_template_child_private (widget_class, PhoshAppGridButton, popover);
