@@ -15,6 +15,7 @@
 #include "shell.h"
 #include "phosh-enums.h"
 #include "osk-manager.h"
+#include "style-manager.h"
 #include "feedback-manager.h"
 #include "util.h"
 
@@ -76,6 +77,9 @@ struct _PhoshHome
   GtkGesture     *osk_toggle_long_press; /* to toggle osk from the home bar itself */
   GSettings      *phosh_settings;
 
+  PhoshMonitor    *monitor;
+  PhoshBackground *background;
+  gboolean         use_background;
 };
 G_DEFINE_TYPE(PhoshHome, phosh_home, PHOSH_TYPE_DRAG_SURFACE);
 
@@ -83,10 +87,15 @@ G_DEFINE_TYPE(PhoshHome, phosh_home, PHOSH_TYPE_DRAG_SURFACE);
 static void
 phosh_home_update_home_bar (PhoshHome *self)
 {
-  gboolean reveal;
+  gboolean reveal, solid = TRUE;
+  PhoshDragSurfaceState drag_state = phosh_drag_surface_get_drag_state (PHOSH_DRAG_SURFACE (self));
 
   reveal = !(self->state == PHOSH_HOME_STATE_UNFOLDED);
   gtk_revealer_set_reveal_child (GTK_REVEALER (self->rev_powerbar), reveal);
+
+  if (self->use_background)
+    solid = !!(self->state == PHOSH_HOME_STATE_FOLDED && drag_state != PHOSH_DRAG_SURFACE_STATE_DRAGGED);
+  phosh_util_toggle_style_class (GTK_WIDGET (self), "p-solid", solid);
 }
 
 
@@ -136,7 +145,7 @@ phosh_home_get_property (GObject    *object,
 
 
 static void
-update_drag_handle (PhoshHome *self, gboolean commit)
+update_drag_handle (PhoshHome *self, gboolean queue_draw)
 {
   gboolean success;
   gint handle = 0;
@@ -166,8 +175,9 @@ update_drag_handle (PhoshHome *self, gboolean commit)
 
   g_debug ("Drag Handle: %d", handle);
   phosh_drag_surface_set_drag_handle (PHOSH_DRAG_SURFACE (self), handle);
-  if (commit)
-    phosh_layer_surface_wl_surface_commit (PHOSH_LAYER_SURFACE (self));
+  /* Trigger redraw and surface commit */
+  if (queue_draw)
+    gtk_widget_queue_draw (GTK_WIDGET (self));
 }
 
 
@@ -194,14 +204,24 @@ on_configure_event (PhoshHome *self, GdkEventConfigure *event)
   /* If the size changes we need to update the folded margin */
   phosh_drag_surface_set_margin (PHOSH_DRAG_SURFACE (self), margin, 0);
   /* Update drag handle since overview size might have changed */
-  update_drag_handle (self, FALSE);
-  /* Trigger redraw and surface commit */
-  gtk_widget_queue_draw (GTK_WIDGET (self));
+  update_drag_handle (self, TRUE);
 
   return FALSE;
 }
 
 
+static void
+phosh_home_map (GtkWidget *widget)
+{
+  PhoshHome *self = PHOSH_HOME (widget);
+
+  GTK_WIDGET_CLASS (phosh_home_parent_class)->map (widget);
+
+  phosh_layer_surface_set_stacked_below (PHOSH_LAYER_SURFACE (self->background),
+                                         PHOSH_LAYER_SURFACE (self));
+}
+
+
 static void
 on_home_released (GtkButton *button, int n_press, double x, double y, GtkGestureMultiPress *gesture)
 {
@@ -282,14 +302,13 @@ fold_cb (PhoshHome *self, PhoshOverview *overview)
 }
 
 
-static gboolean
+static void
 delayed_handle_resize (gpointer data)
 {
   PhoshHome *self = PHOSH_HOME (data);
 
   self->debounce_handle = 0;
   update_drag_handle (self, TRUE);
-  return G_SOURCE_REMOVE;
 }
 
 
@@ -301,7 +320,7 @@ on_has_activities_changed (PhoshHome *self)
   /* TODO: we need to debounce the handle resize a little until all
      the queued resizing is done, would be nicer to have that tied to
      a signal */
-  self->debounce_handle = g_timeout_add (200, delayed_handle_resize, self);
+  self->debounce_handle = g_timeout_add_once (200, delayed_handle_resize, self);
   g_source_set_name_by_id (self->debounce_handle, "[phosh] delayed_handle_resize");
 }
 
@@ -367,6 +386,7 @@ toggle_application_view_action (GSimpleAction *action, GVariant *param, gpointer
     self->focus_app_search = TRUE;
 }
 
+
 static void
 add_keybindings (PhoshHome *self)
 {
@@ -427,6 +447,31 @@ on_keybindings_changed (PhoshHome *self,
 }
 
 
+static void
+phosh_home_set_background_alpha (PhoshHome *self, double alpha)
+{
+  if (self->background)
+    phosh_layer_surface_set_alpha (PHOSH_LAYER_SURFACE (self->background), alpha);
+}
+
+
+static void
+phosh_home_dragged (PhoshDragSurface *drag_surface, int margin)
+{
+  PhoshHome *self = PHOSH_HOME (drag_surface);
+  int width, height;
+  double progress, alpha;
+
+  gtk_window_get_size (GTK_WINDOW (self), &width, &height);
+  progress = 1.0 - (-margin / (double)(height - PHOSH_HOME_BAR_HEIGHT));
+  /* Avoid negative values when resizing the surface */
+  progress = MAX (0, progress);
+
+  alpha = hdy_ease_out_cubic (progress);
+  phosh_home_set_background_alpha (self, alpha);
+}
+
+
 static void
 on_drag_state_changed (PhoshHome *self)
 {
@@ -444,11 +489,14 @@ on_drag_state_changed (PhoshHome *self)
       phosh_overview_focus_app_search (PHOSH_OVERVIEW (self->overview));
       self->focus_app_search = FALSE;
     }
+    phosh_home_set_background_alpha (self, 1.0);
     break;
   case PHOSH_DRAG_SURFACE_STATE_FOLDED:
     state = PHOSH_HOME_STATE_FOLDED;
+    phosh_home_set_background_alpha (self, 0.0);
     break;
   case PHOSH_DRAG_SURFACE_STATE_DRAGGED:
+    state = PHOSH_HOME_STATE_TRANSITION;
     if (self->state == PHOSH_HOME_STATE_FOLDED)
       phosh_overview_reset (PHOSH_OVERVIEW (self->overview));
     break;
@@ -465,8 +513,47 @@ on_drag_state_changed (PhoshHome *self)
   phosh_home_update_home_bar (self);
 
   phosh_layer_surface_set_kbd_interactivity (PHOSH_LAYER_SURFACE (self), kbd_interactivity);
-  update_drag_handle (self, FALSE);
-  gtk_widget_queue_draw (GTK_WIDGET (self));
+  update_drag_handle (self, TRUE);
+}
+
+
+static void
+phosh_home_add_background (PhoshHome *self)
+{
+  PhoshWayland *wl = phosh_wayland_get_default ();
+  PhoshShell *shell = phosh_shell_get_default ();
+  PhoshMonitor *monitor;
+  cairo_rectangle_int_t rect = { 0, 0, 0, 0 };
+  cairo_region_t *region;
+
+  monitor = phosh_shell_get_primary_monitor (shell);
+  self->background = PHOSH_BACKGROUND (phosh_background_new (
+                                         phosh_wayland_get_zwlr_layer_shell_v1(wl),
+                                         monitor,
+                                         /* Span over whole display */
+                                         FALSE,
+                                         ZWLR_LAYER_SHELL_V1_LAYER_TOP));
+  gtk_widget_show (GTK_WIDGET (self->background));
+  g_signal_connect_object (phosh_shell_get_background_manager (shell),
+                           "config-changed",
+                           G_CALLBACK (phosh_background_needs_update),
+                           self->background,
+                           G_CONNECT_SWAPPED);
+
+  region = cairo_region_create_rectangle(&rect);
+  gtk_widget_input_shape_combine_region (GTK_WIDGET (self->background), region);
+  cairo_region_destroy (region);
+}
+
+
+static void
+on_theme_name_changed (PhoshHome  *self, GParamSpec *pspec, PhoshStyleManager *style_manager)
+{
+  g_assert (PHOSH_IS_HOME (self));
+  g_assert (PHOSH_IS_STYLE_MANAGER (style_manager));
+
+  self->use_background = !phosh_style_manager_is_high_contrast (style_manager);
+  phosh_home_update_home_bar (self);
 }
 
 
@@ -474,6 +561,7 @@ static void
 phosh_home_constructed (GObject *object)
 {
   PhoshHome *self = PHOSH_HOME (object);
+  PhoshShell *shell = phosh_shell_get_default ();
   PhoshOskManager *osk_manager;
 
   G_OBJECT_CLASS (phosh_home_parent_class)->constructed (object);
@@ -495,6 +583,14 @@ phosh_home_constructed (GObject *object)
 
   g_object_set_data (G_OBJECT (self->click_gesture), "phosh-home", self);
   g_object_set_data (G_OBJECT (self->osk_toggle_long_press), "phosh-home", self);
+
+  phosh_home_add_background (self);
+  g_signal_connect_object (phosh_shell_get_style_manager (shell),
+                           "notify::theme-name",
+                           G_CALLBACK (on_theme_name_changed),
+                           self,
+                           G_CONNECT_SWAPPED);
+  on_theme_name_changed (self, NULL, phosh_shell_get_style_manager (shell));
 }
 
 
@@ -512,6 +608,8 @@ phosh_home_dispose (GObject *object)
   }
   g_clear_handle_id (&self->debounce_handle, g_source_remove);
 
+  g_clear_pointer (&self->background, phosh_cp_widget_destroy);
+
   G_OBJECT_CLASS (phosh_home_parent_class)->dispose (object);
 }
 
@@ -521,6 +619,7 @@ phosh_home_class_init (PhoshHomeClass *klass)
 {
   GObjectClass *object_class = (GObjectClass *)klass;
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
+  PhoshDragSurfaceClass *drag_surface_class = PHOSH_DRAG_SURFACE_CLASS (klass);
 
   object_class->constructed = phosh_home_constructed;
   object_class->dispose = phosh_home_dispose;
@@ -528,6 +627,10 @@ phosh_home_class_init (PhoshHomeClass *klass)
   object_class->set_property = phosh_home_set_property;
   object_class->get_property = phosh_home_get_property;
 
+  widget_class->map = phosh_home_map;
+
+  drag_surface_class->dragged = phosh_home_dragged;
+
   /**
    * PhoshHome:state:
    *
@@ -555,7 +658,7 @@ phosh_home_class_init (PhoshHomeClass *klass)
   g_type_ensure (PHOSH_TYPE_OVERVIEW);
 
   gtk_widget_class_set_template_from_resource (widget_class,
-                                               "/sm/puri/phosh/ui/home.ui");
+                                               "/mobi/phosh/ui/home.ui");
   gtk_widget_class_bind_template_child (widget_class, PhoshHome, click_gesture);
   gtk_widget_class_bind_template_child (widget_class, PhoshHome, home_bar);
   gtk_widget_class_bind_template_child (widget_class, PhoshHome, osk_toggle_long_press);
@@ -581,6 +684,7 @@ phosh_home_init (PhoshHome *self)
 
   gtk_widget_init_template (GTK_WIDGET (self));
 
+  self->use_background = TRUE;
   self->state = PHOSH_HOME_STATE_FOLDED;
   self->settings = g_settings_new (KEYBINDINGS_SCHEMA_ID);
 
