@@ -185,10 +185,13 @@ on_poll_position_done (GDBusProxy *proxy, GAsyncResult *res, gpointer user_data)
   g_autoptr (GError) err = NULL;
   g_autoptr (GVariant) var = NULL;
 
+  var = g_dbus_proxy_call_finish (proxy, res, &err);
+  if (!var && g_error_matches (err, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+    return;
+
   g_return_if_fail (PHOSH_IS_MEDIA_PLAYER (user_data));
   self = PHOSH_MEDIA_PLAYER (user_data);
 
-  var = g_dbus_proxy_call_finish (proxy, res, &err);
   if (err) {
     g_warning ("Could not get Position from MPRIS player, hiding box_pos_len: %s", err->message);
     self->track_position = -1;
@@ -671,7 +674,7 @@ phosh_media_player_class_init (PhoshMediaPlayerClass *klass)
 
   gtk_widget_class_set_css_name (widget_class, "phosh-media-player");
   gtk_widget_class_set_template_from_resource (widget_class,
-                                               "/sm/puri/phosh/ui/media-player.ui");
+                                               "/mobi/phosh/ui/media-player.ui");
   gtk_widget_class_bind_template_child (widget_class, PhoshMediaPlayer, btn_next);
   gtk_widget_class_bind_template_child (widget_class, PhoshMediaPlayer, btn_play);
   gtk_widget_class_bind_template_child (widget_class, PhoshMediaPlayer, btn_prev);
