@@ -45,6 +45,8 @@ struct _PhoshNotificationFrame {
   GBinding *bind_icon;
   GBinding *bind_timestamp;
 
+  GtkRevealer *revealer;
+
   GtkWidget *box;
   GtkWidget *lbl_app_name;
   GtkWidget *img_icon;
@@ -80,9 +82,9 @@ static guint signals[N_SIGNALS] = { 0 };
 
 static void
 phosh_notification_frame_set_property (GObject      *object,
-                                        guint         property_id,
-                                        const GValue *value,
-                                        GParamSpec   *pspec)
+                                       guint         property_id,
+                                       const GValue *value,
+                                       GParamSpec   *pspec)
 {
   PhoshNotificationFrame *self = PHOSH_NOTIFICATION_FRAME (object);
 
@@ -102,9 +104,9 @@ phosh_notification_frame_set_property (GObject      *object,
 
 static void
 phosh_notification_frame_get_property (GObject    *object,
-                                        guint       property_id,
-                                        GValue     *value,
-                                        GParamSpec *pspec)
+                                       guint       property_id,
+                                       GValue     *value,
+                                       GParamSpec *pspec)
 {
   PhoshNotificationFrame *self = PHOSH_NOTIFICATION_FRAME (object);
 
@@ -273,9 +275,21 @@ notification_activated (PhoshNotificationFrame *self,
 
 static void
 removed (PhoshNotificationFrame *self)
+{
+  gtk_revealer_set_reveal_child (GTK_REVEALER (self->revealer), FALSE);
+}
+
+
+static void
+on_child_revealed_changed (PhoshNotificationFrame *self,
+                           GParamSpec             *pspec,
+                           GtkRevealer            *revealer)
 {
   guint i, n;
 
+  if (gtk_revealer_get_child_revealed (revealer))
+    return;
+
   n = g_list_model_get_n_items (self->model);
   for (i = 0; i < n; i++) {
     g_autoptr (PhoshNotification) notification = NULL;
@@ -287,22 +301,48 @@ removed (PhoshNotificationFrame *self)
 }
 
 
+/**
+ * needs_separator:
+ * @row: the #GtkListBoxRow
+ *
+ * Checks if a visible separator is needed. This is only the case when
+ * there are no action buttons as otherwise the action buttons are enough
+ * separation.
+ *
+ * Returns: `TRUE` if a separator is needed
+ */
+static gboolean
+needs_separator (PhoshNotificationContent *content)
+{
+  PhoshNotification *notification;
+  GStrv actions;
+
+  notification = phosh_notification_content_get_notification (content);
+  actions = phosh_notification_get_actions (notification);
+
+  if (gm_strv_is_null_or_empty (actions))
+    return TRUE;
+
+  if (g_strv_length (actions) == 2 && g_str_equal (actions[0], "default"))
+    return TRUE;
+
+  return FALSE;
+}
+
+
 static void
 header_func (GtkListBoxRow *current_row, GtkListBoxRow *prev_row, gpointer user_data)
 {
-  PhoshNotificationContent *prev_content;
-  PhoshNotification *prev_notification;
-  GStrv prev_actions;
+  PhoshNotificationContent *content;
 
-  if (prev_row == NULL)
+  if (prev_row == NULL) {
+    gtk_list_box_row_set_header (current_row, NULL);
     return;
+  }
 
-  prev_content = PHOSH_NOTIFICATION_CONTENT (prev_row);
-  prev_notification = phosh_notification_content_get_notification (prev_content);
-  prev_actions = phosh_notification_get_actions (prev_notification);
+  content = PHOSH_NOTIFICATION_CONTENT (prev_row);
 
-  /* only add separator if the previous notification doesn't have actions */
-  if (gm_strv_is_null_or_empty (prev_actions)) {
+  if (needs_separator (content)) {
     GtkWidget *separator = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
     gtk_widget_set_hexpand (separator, TRUE);
     gtk_list_box_row_set_header (current_row, separator);
@@ -363,7 +403,8 @@ phosh_notification_frame_class_init (PhoshNotificationFrameClass *klass)
   g_type_ensure (PHOSH_TYPE_TIMESTAMP_LABEL);
 
   gtk_widget_class_set_template_from_resource (widget_class,
-                                               "/sm/puri/phosh/ui/notification-frame.ui");
+                                               "/mobi/phosh/ui/notification-frame.ui");
+  gtk_widget_class_bind_template_child (widget_class, PhoshNotificationFrame, revealer);
   gtk_widget_class_bind_template_child (widget_class, PhoshNotificationFrame, box);
   gtk_widget_class_bind_template_child (widget_class, PhoshNotificationFrame, lbl_app_name);
   gtk_widget_class_bind_template_child (widget_class, PhoshNotificationFrame, img_icon);
@@ -398,6 +439,9 @@ phosh_notification_frame_init (PhoshNotificationFrame *self)
                                 header_func,
                                 self,
                                 NULL);
+
+  g_signal_connect_swapped (self->revealer, "notify::child-revealed",
+                            G_CALLBACK (on_child_revealed_changed), self);
 }
 
 
@@ -529,3 +573,4 @@ phosh_notification_frame_get_action_filter_keys (PhoshNotificationFrame *self)
 
   return (const char *const *)self->action_filter_keys;
 }
+
