@@ -25,6 +25,7 @@
 
 #include "phosh-config.h"
 #include "ambient.h"
+#include "background.h"
 #include "drag-surface.h"
 #include "shell.h"
 #include "app-tracker.h"
@@ -34,6 +35,7 @@
 #include "bt-manager.h"
 #include "connectivity-info.h"
 #include "calls-manager.h"
+#include "cell-broadcast-manager.h"
 #include "docked-info.h"
 #include "docked-manager.h"
 #include "emergency-calls-manager.h"
@@ -80,9 +82,11 @@
 #include "screenshot-manager.h"
 #include "session-manager.h"
 #include "splash-manager.h"
+#include "style-manager.h"
 #include "suspend-manager.h"
 #include "system-prompter.h"
 #include "top-panel.h"
+#include "top-panel-bg.h"
 #include "torch-manager.h"
 #include "torch-info.h"
 #include "util.h"
@@ -172,7 +176,9 @@ typedef struct
   PhoshEmergencyCallsManager *emergency_calls_manager;
   PhoshPowerMenuManager *power_menu_manager;
   PhoshLayoutManager *layout_manager;
+  PhoshStyleManager *style_manager;
   PhoshLauncherEntryManager *launcher_entry_manager;
+  PhoshCellBroadcastManager *cell_broadcast_manager;
 
   /* sensors */
   PhoshSensorProxyManager *sensor_proxy_manager;
@@ -193,9 +199,6 @@ typedef struct
 
   PhoshShellStateFlags shell_state;
 
-  char           *theme_name;
-  GtkCssProvider *css_provider;
-
   GSettings      *settings;
 } PhoshShellPrivate;
 
@@ -261,8 +264,7 @@ update_top_level_layer (PhoshShell *self)
     return;
 
   g_debug ("Moving top-panel to %s layer", use_top_layer ? "top" : "overlay");
-  phosh_layer_surface_set_layer (PHOSH_LAYER_SURFACE (priv->top_panel), layer);
-  phosh_layer_surface_wl_surface_commit (PHOSH_LAYER_SURFACE (priv->top_panel));
+  phosh_top_panel_set_layer (PHOSH_TOP_PANEL (priv->top_panel), layer);
 }
 
 
@@ -301,6 +303,10 @@ on_home_state_changed (PhoshShell *self, GParamSpec *pspec, PhoshHome *home)
   priv = phosh_shell_get_instance_private (self);
 
   state = phosh_home_get_state (PHOSH_HOME (priv->home));
+
+  phosh_top_panel_set_bar_transparent (PHOSH_TOP_PANEL (priv->top_panel),
+                                       (state != PHOSH_HOME_STATE_FOLDED));
+
   phosh_shell_set_state (self, PHOSH_STATE_OVERVIEW, state == PHOSH_HOME_STATE_UNFOLDED);
 }
 
@@ -353,6 +359,7 @@ setup_primary_monitor_signal_handlers (PhoshShell *self)
     on_primary_monitor_configured (self, priv->primary_monitor);
 }
 
+
 static void
 panels_create (PhoshShell *self)
 {
@@ -366,6 +373,8 @@ panels_create (PhoshShell *self)
   g_return_if_fail (monitor);
 
   top_layer = priv->locked ? ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY : ZWLR_LAYER_SHELL_V1_LAYER_TOP;
+
+  /* Top panel */
   priv->top_panel = PHOSH_DRAG_SURFACE (phosh_top_panel_new (
                                           phosh_wayland_get_zwlr_layer_shell_v1 (wl),
                                           phosh_wayland_get_zphoc_layer_shell_effects_v1 (wl),
@@ -373,6 +382,7 @@ panels_create (PhoshShell *self)
                                           top_layer));
   gtk_widget_show (GTK_WIDGET (priv->top_panel));
 
+  /* Home is created after the top-panel so it honors its exclusive zone */
   priv->home = PHOSH_DRAG_SURFACE (phosh_home_new (phosh_wayland_get_zwlr_layer_shell_v1 (wl),
                                                    phosh_wayland_get_zphoc_layer_shell_effects_v1 (wl),
                                                    monitor));
@@ -412,37 +422,6 @@ panels_dispose (PhoshShell *self)
 }
 
 
-/* Select proper style sheet in case of high contrast */
-static void
-on_gtk_theme_name_changed (PhoshShell *self, GParamSpec *pspec, GtkSettings *settings)
-{
-  const char *style;
-  g_autofree char *name = NULL;
-  PhoshShellPrivate *priv = phosh_shell_get_instance_private (self);
-  g_autoptr (GtkCssProvider) provider = gtk_css_provider_new ();
-
-  g_object_get (settings, "gtk-theme-name", &name, NULL);
-
-  if (g_strcmp0 (priv->theme_name, name) == 0)
-    return;
-
-  priv->theme_name = g_steal_pointer (&name);
-  g_debug ("GTK theme: %s", priv->theme_name);
-
-  if (priv->css_provider) {
-    gtk_style_context_remove_provider_for_screen(gdk_screen_get_default (),
-                                                 GTK_STYLE_PROVIDER (priv->css_provider));
-  }
-
-  style = phosh_util_get_stylesheet (priv->theme_name);
-  gtk_css_provider_load_from_resource (provider, style);
-  gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
-                                             GTK_STYLE_PROVIDER (provider),
-                                             GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-  g_set_object (&priv->css_provider, provider);
-}
-
-
 static void
 set_locked (PhoshShell *self, gboolean locked)
 {
@@ -538,6 +517,7 @@ phosh_shell_dispose (GObject *object)
   g_clear_object (&priv->notification_banner);
 
   /* dispose managers in opposite order of declaration */
+  g_clear_object (&priv->cell_broadcast_manager);
   g_clear_object (&priv->launcher_entry_manager);
   g_clear_object (&priv->power_menu_manager);
   g_clear_object (&priv->emergency_calls_manager);
@@ -575,6 +555,7 @@ phosh_shell_dispose (GObject *object)
   g_clear_object (&priv->app_tracker);
   g_clear_object (&priv->suspend_manager);
   g_clear_object (&priv->layout_manager);
+  g_clear_object (&priv->style_manager);
 
   /* sensors */
   g_clear_object (&priv->proximity);
@@ -584,9 +565,6 @@ phosh_shell_dispose (GObject *object)
   phosh_system_prompter_unregister ();
   g_clear_object (&priv->session_manager);
 
-  g_clear_pointer (&priv->theme_name, g_free);
-  g_clear_object (&priv->css_provider);
-
   g_clear_object (&priv->action_map);
   g_clear_object (&priv->settings);
 
@@ -711,18 +689,18 @@ notify_compositor_up_state (PhoshShell *self, enum phosh_private_shell_state sta
 }
 
 
-static gboolean
-on_startup_finished (PhoshShell *self)
+static void
+on_startup_finished (gpointer data)
 {
+  PhoshShell *self = PHOSH_SHELL (data);
   PhoshShellPrivate *priv;
 
-  g_return_val_if_fail (PHOSH_IS_SHELL (self), G_SOURCE_REMOVE);
+  g_return_if_fail (PHOSH_IS_SHELL (self));
   priv = phosh_shell_get_instance_private (self);
 
   notify_compositor_up_state (self, PHOSH_PRIVATE_SHELL_STATE_UP);
 
   priv->startup_finished_id = 0;
-  return G_SOURCE_REMOVE;
 }
 
 
@@ -741,20 +719,20 @@ setup_idle_cb (PhoshShell *self)
     g_message ("Failed to connect to sensor-proxy: %s", err->message);
 
   priv->layout_manager = phosh_layout_manager_new ();
-  panels_create (self);
-  /* Create background after panel since it needs the panel's size */
+  /* PhoshHome needs the background manager */
   priv->background_manager = phosh_background_manager_new ();
+  panels_create (self);
 
   g_signal_connect_object (priv->toplevel_manager,
                            "notify::num-toplevels",
-                           G_CALLBACK(on_num_toplevels_changed),
+                           G_CALLBACK (on_num_toplevels_changed),
                            self,
                            G_CONNECT_SWAPPED);
   on_num_toplevels_changed (self, NULL, priv->toplevel_manager);
 
   g_signal_connect_object (priv->toplevel_manager,
                            "toplevel-added",
-                           G_CALLBACK(on_toplevel_added),
+                           G_CALLBACK (on_toplevel_added),
                            self,
                            G_CONNECT_SWAPPED);
 
@@ -799,18 +777,19 @@ setup_idle_cb (PhoshShell *self)
   priv->gnome_shell_manager = phosh_gnome_shell_manager_get_default ();
   priv->screenshot_manager = phosh_screenshot_manager_new ();
   priv->splash_manager = phosh_splash_manager_new (priv->app_tracker);
-  priv->run_command_manager = phosh_run_command_manager_new();
+  priv->run_command_manager = phosh_run_command_manager_new ();
   priv->network_auth_manager = phosh_network_auth_manager_new ();
   priv->portal_access_manager = phosh_portal_access_manager_new ();
   priv->suspend_manager = phosh_suspend_manager_new ();
   priv->emergency_calls_manager = phosh_emergency_calls_manager_new ();
   priv->power_menu_manager = phosh_power_menu_manager_new ();
+  priv->cell_broadcast_manager = phosh_cell_broadcast_manager_new ();
 
   setup_primary_monitor_signal_handlers (self);
 
   /* Delay signaling to the compositor a bit so that idle handlers get a chance to run and
      the user can unlock right away. Ideally we'd not need this */
-  priv->startup_finished_id = g_timeout_add_seconds (1, (GSourceFunc)on_startup_finished, self);
+  priv->startup_finished_id = g_timeout_add_seconds_once (1, on_startup_finished, self);
   g_source_set_name_by_id (priv->startup_finished_id, "[PhoshShell] startup finished");
 
   priv->startup_finished = TRUE;
@@ -1042,7 +1021,7 @@ phosh_shell_constructed (GObject *object)
   }
 
   gtk_icon_theme_add_resource_path (gtk_icon_theme_get_default (),
-                                    "/sm/puri/phosh/icons");
+                                    "/mobi/phosh/icons");
 
   priv->calls_manager = phosh_calls_manager_new ();
   priv->launcher_entry_manager = phosh_launcher_entry_manager_new ();
@@ -1305,7 +1284,6 @@ static void
 phosh_shell_init (PhoshShell *self)
 {
   PhoshShellPrivate *priv = phosh_shell_get_instance_private (self);
-  GtkSettings *gtk_settings;
 
   cui_init (TRUE);
 
@@ -1316,15 +1294,7 @@ phosh_shell_init (PhoshShell *self)
                                       debug_keys,
                                       G_N_ELEMENTS (debug_keys));
 
-  gtk_settings = gtk_settings_get_default ();
-  g_object_set (G_OBJECT (gtk_settings), "gtk-application-prefer-dark-theme", TRUE, NULL);
-
-  g_signal_connect_swapped (gtk_settings,
-                            "notify::gtk-theme-name",
-                            G_CALLBACK (on_gtk_theme_name_changed),
-                            self);
-  on_gtk_theme_name_changed (self, NULL, gtk_settings);
-
+  priv->style_manager = phosh_style_manager_new ();
   priv->shell_state = PHOSH_STATE_SETTINGS;
   priv->action_map = g_simple_action_group_new ();
 }
@@ -1661,6 +1631,25 @@ phosh_shell_get_mode_manager (PhoshShell *self)
   return priv->mode_manager;
 }
 
+/**
+ * phosh_shell_get_style_manager:
+ * @self: The shell singleton
+ *
+ * Get the style manager
+ *
+ * Returns: (transfer none): The style manager
+ */
+PhoshStyleManager *
+phosh_shell_get_style_manager (PhoshShell *self)
+{
+  PhoshShellPrivate *priv;
+
+  g_return_val_if_fail (PHOSH_IS_SHELL (self), NULL);
+  priv = phosh_shell_get_instance_private (self);
+
+  g_assert (PHOSH_IS_STYLE_MANAGER (priv->style_manager));
+  return priv->style_manager;
+}
 
 /**
  * phosh_shell_get_monitor_manager:
@@ -2484,7 +2473,6 @@ phosh_shell_get_blanked (PhoshShell *self)
   return phosh_shell_get_state (self) & PHOSH_STATE_BLANKED;
 }
 
-
 /**
  * phosh_shell_activate_action:
  * @self: The #PhoshShell singleton
