@@ -18,7 +18,7 @@
  *
  * Additional status information associated with a [class@QuickSetting].
  *
- * This is displayed when the quick setting is long pressed.
+ * This is displayed when the quick setting needs to show status.
  */
 
 enum {
@@ -48,7 +48,7 @@ typedef struct {
   GtkWidget    *footer_widget;
 } PhoshStatusPagePrivate;
 
-G_DEFINE_TYPE_WITH_PRIVATE (PhoshStatusPage, phosh_status_page, GTK_TYPE_REVEALER);
+G_DEFINE_TYPE_WITH_PRIVATE (PhoshStatusPage, phosh_status_page, GTK_TYPE_BIN);
 
 
 static void
@@ -225,7 +225,7 @@ phosh_status_page_class_init (PhoshStatusPageClass *klass)
   g_object_class_install_properties (object_class, PROP_LAST_PROP, props);
 
   g_type_ensure (PHOSH_TYPE_STATUS_PAGE_PLACEHOLDER);
-  gtk_widget_class_set_template_from_resource (widget_class, "/sm/puri/phosh/ui/status-page.ui");
+  gtk_widget_class_set_template_from_resource (widget_class, "/mobi/phosh/ui/status-page.ui");
 
   gtk_widget_class_bind_template_child_private (widget_class, PhoshStatusPage, content_bin);
   gtk_widget_class_bind_template_child_private (widget_class, PhoshStatusPage, footer_bin);
