@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2018-2022 Purism SPC
+ *               2023-2024 The Phosh Developers
  *
  * SPDX-License-Identifier: GPL-3.0-or-later
  *
@@ -19,10 +20,13 @@
 #include "session-manager.h"
 #include "settings.h"
 #include "top-panel.h"
+#include "top-panel-bg.h"
 #include "arrow.h"
 #include "util.h"
 #include "wall-clock.h"
 
+#include <handy.h>
+
 #define GNOME_DESKTOP_USE_UNSTABLE_API
 #include <libgnome-desktop/gnome-xkb-info.h>
 
@@ -74,6 +78,7 @@ typedef struct _PhoshTopPanel {
   GtkWidget *stack;
   GtkWidget *arrow;
 
+  GtkWidget *top_bar_bin;
   GtkWidget *box_top_bar;
   GtkWidget *lbl_clock;      /* top-bar clock */
   GtkWidget *lbl_lang;
@@ -93,6 +98,8 @@ typedef struct _PhoshTopPanel {
   GSettings      *kb_settings;
 
   GtkGesture     *click_gesture; /* needed so that the gesture isn't destroyed immediately */
+
+  PhoshTopPanelBg *background;
 } PhoshTopPanel;
 
 G_DEFINE_TYPE (PhoshTopPanel, phosh_top_panel, PHOSH_TYPE_DRAG_SURFACE)
@@ -139,7 +146,7 @@ phosh_top_panel_get_property (GObject    *object,
 
 
 static void
-update_drag_handle (PhoshTopPanel *self, gboolean commit)
+update_drag_handle (PhoshTopPanel *self, gboolean queue_draw)
 {
   int handle, offset;
 
@@ -159,8 +166,9 @@ update_drag_handle (PhoshTopPanel *self, gboolean commit)
                                     PHOSH_DRAG_SURFACE_DRAG_MODE_HANDLE);
   phosh_drag_surface_set_drag_handle (PHOSH_DRAG_SURFACE (self), handle);
 
-  if (commit)
-    phosh_layer_surface_wl_surface_commit (PHOSH_LAYER_SURFACE (self));
+  /* Trigger redraw and surface commit */
+  if (queue_draw)
+    gtk_widget_queue_draw (GTK_WIDGET (self));
 }
 
 
@@ -353,29 +361,6 @@ on_input_setting_changed (PhoshTopPanel *self,
 }
 
 
-static gboolean
-on_key_press_event (PhoshTopPanel *self, GdkEventKey *event, gpointer data)
-{
-  gboolean handled = FALSE;
-
-  g_return_val_if_fail (PHOSH_IS_TOP_PANEL (self), FALSE);
-
-  if (!self->settings)
-    return handled;
-
-  switch (event->keyval) {
-    case GDK_KEY_Escape:
-      phosh_top_panel_fold (self);
-      handled = TRUE;
-      break;
-    default:
-      /* nothing to do */
-      break;
-    }
-  return handled;
-}
-
-
 static void
 released_cb (PhoshTopPanel *self, int n_press, double x, double y, GtkGestureMultiPress *gesture)
 {
@@ -438,14 +423,27 @@ on_keybindings_changed (PhoshTopPanel *self,
 }
 
 
+static double
+ease_in_quintic (double p)
+{
+  return p * p * p * p * p;
+}
+
+
 static void
 phosh_top_panel_dragged (PhoshDragSurface *drag_surface, int margin)
 {
   PhoshTopPanel *self = PHOSH_TOP_PANEL (drag_surface);
   int width, height;
+  double progress, transparency;
+
   gtk_window_get_size (GTK_WINDOW (self), &width, &height);
-  phosh_arrow_set_progress (PHOSH_ARROW (self->arrow), -margin / (double)(height - PHOSH_TOP_BAR_HEIGHT));
-  g_debug ("Margin: %d", margin);
+  progress = -margin / (double)(height - PHOSH_TOP_BAR_HEIGHT);
+  phosh_arrow_set_progress (PHOSH_ARROW (self->arrow), progress);
+
+  progress = MIN (1.0, progress);
+  transparency = ease_in_quintic (progress);
+  phosh_top_panel_bg_set_transparency (self->background, transparency);
 }
 
 
@@ -455,7 +453,7 @@ on_drag_state_changed (PhoshTopPanel *self)
   PhoshTopPanelState state = self->state;
   const char *visible;
   gboolean kbd_interactivity = FALSE;
-  double arrow = -1.0;
+  double progress = -1.0;
 
   /* Close the popover on any drag */
   gtk_widget_hide (self->menu_system);
@@ -466,16 +464,18 @@ on_drag_state_changed (PhoshTopPanel *self)
     update_drag_handle (self, TRUE);
     kbd_interactivity = TRUE;
     visible = "arrow";
-    arrow = 0.0;
+    progress = 0.0;
+    phosh_top_panel_bg_set_transparency (self->background, 0.0);
     break;
   case PHOSH_DRAG_SURFACE_STATE_FOLDED:
     state = PHOSH_TOP_PANEL_STATE_FOLDED;
     visible = "top-bar";
-    arrow = 1.0;
+    progress = 1.0;
+    phosh_top_panel_bg_set_transparency (self->background, 1.0);
     break;
   case PHOSH_DRAG_SURFACE_STATE_DRAGGED:
     visible = "arrow";
-    arrow = phosh_arrow_get_progress (PHOSH_ARROW (self->arrow));
+    progress = phosh_arrow_get_progress (PHOSH_ARROW (self->arrow));
     break;
   default:
     g_return_if_reached ();
@@ -483,7 +483,7 @@ on_drag_state_changed (PhoshTopPanel *self)
 
   g_debug ("%s: state: %d, visible: %s", __func__, self->state, visible);
   gtk_stack_set_visible_child_name (GTK_STACK (self->stack), visible);
-  phosh_arrow_set_progress (PHOSH_ARROW (self->arrow), arrow);
+  phosh_arrow_set_progress (PHOSH_ARROW (self->arrow), progress);
 
   if (self->state != state) {
     self->state = state;
@@ -495,6 +495,55 @@ on_drag_state_changed (PhoshTopPanel *self)
 }
 
 
+static void
+phosh_top_panel_map (GtkWidget *widget)
+{
+  PhoshTopPanel *self = PHOSH_TOP_PANEL (widget);
+
+  GTK_WIDGET_CLASS (phosh_top_panel_parent_class)->map (widget);
+
+  phosh_layer_surface_set_stacked_below (PHOSH_LAYER_SURFACE (self->background),
+                                         PHOSH_LAYER_SURFACE (self));
+}
+
+
+static void
+phosh_top_panel_add_background (PhoshTopPanel *self)
+{
+  PhoshWayland *wl = phosh_wayland_get_default ();
+  PhoshShell *shell = phosh_shell_get_default ();
+  PhoshMonitor *monitor;
+  guint32 layer;
+  cairo_rectangle_int_t rect = { 0, 0, 0, 0 };
+  cairo_region_t *region;
+
+  monitor = phosh_shell_get_primary_monitor (shell);
+  layer = phosh_layer_surface_get_layer (PHOSH_LAYER_SURFACE (self));
+  self->background = phosh_top_panel_bg_new (phosh_wayland_get_zwlr_layer_shell_v1 (wl),
+                                             monitor,
+                                             layer);
+
+  gtk_widget_show (GTK_WIDGET (self->background));
+
+  region = cairo_region_create_rectangle (&rect);
+  gtk_widget_input_shape_combine_region (GTK_WIDGET (self->background), region);
+  cairo_region_destroy (region);
+}
+
+
+static void
+on_theme_name_changed (PhoshTopPanel  *self, GParamSpec *pspec, PhoshStyleManager *style_manager)
+{
+  gboolean solid;
+
+  g_assert (PHOSH_IS_TOP_PANEL (self));
+  g_assert (PHOSH_IS_STYLE_MANAGER (style_manager));
+
+  solid = phosh_style_manager_is_high_contrast (style_manager);
+  phosh_util_toggle_style_class (GTK_WIDGET (self), "p-solid", solid);
+}
+
+
 static GActionEntry entries[] = {
   { .name = "poweroff", .activate = on_shutdown_action },
   { .name = "restart", .activate = on_restart_action },
@@ -510,6 +559,7 @@ phosh_top_panel_constructed (GObject *object)
   PhoshTopPanel *self = PHOSH_TOP_PANEL (object);
   GdkDisplay *display = gdk_display_get_default ();
   PhoshWallClock *wall_clock = phosh_wall_clock_get_default ();
+  PhoshShell *shell = phosh_shell_get_default ();
 
   g_autoptr (GSettings) phosh_settings = g_settings_new ("sm.puri.phosh");
 
@@ -559,13 +609,6 @@ phosh_top_panel_constructed (GObject *object)
                                  NULL, self, NULL);
   }
 
-  /* Settings menu and it's top-panel / menu */
-  gtk_widget_add_events (GTK_WIDGET (self), GDK_ALL_EVENTS_MASK);
-  g_signal_connect (G_OBJECT (self),
-                    "key-press-event",
-                    G_CALLBACK (on_key_press_event),
-                    NULL);
-
   self->actions = g_simple_action_group_new ();
   gtk_widget_insert_action_group (GTK_WIDGET (self), "panel",
                                   G_ACTION_GROUP (self->actions));
@@ -573,14 +616,13 @@ phosh_top_panel_constructed (GObject *object)
                                    entries, G_N_ELEMENTS (entries),
                                    self);
   if (!phosh_shell_started_by_display_manager (phosh_shell_get_default ())) {
-    GAction *action = g_action_map_lookup_action (G_ACTION_MAP (self->actions),
-                                                  "logout");
-    g_simple_action_set_enabled (G_SIMPLE_ACTION(action), FALSE);
+    GAction *action = g_action_map_lookup_action (G_ACTION_MAP (self->actions), "logout");
+    g_simple_action_set_enabled (G_SIMPLE_ACTION (action), FALSE);
   }
 
   g_settings_bind (phosh_settings,
                    "enable-suspend",
-                   g_action_map_lookup_action(G_ACTION_MAP (self->actions), "suspend"),
+                   g_action_map_lookup_action (G_ACTION_MAP (self->actions), "suspend"),
                    "enabled",
                    G_SETTINGS_BIND_GET);
 
@@ -599,6 +641,14 @@ phosh_top_panel_constructed (GObject *object)
   add_keybindings (self);
 
   g_signal_connect (self, "notify::drag-state", G_CALLBACK (on_drag_state_changed), NULL);
+
+  phosh_top_panel_add_background (self);
+  g_signal_connect_object (phosh_shell_get_style_manager (shell),
+                           "notify::theme-name",
+                           G_CALLBACK (on_theme_name_changed),
+                           self,
+                           G_CONNECT_SWAPPED);
+  on_theme_name_changed (self, NULL, phosh_shell_get_style_manager (shell));
 }
 
 
@@ -618,6 +668,7 @@ phosh_top_panel_dispose (GObject *object)
     g_signal_handlers_disconnect_by_data (self->seat, self);
     self->seat = NULL;
   }
+  g_clear_pointer (&self->background, phosh_cp_widget_destroy);
 
   G_OBJECT_CLASS (phosh_top_panel_parent_class)->dispose (object);
 }
@@ -646,9 +697,7 @@ on_configure_event (PhoshTopPanel *self, GdkEventConfigure *event)
   /* If the size changes we need to update the folded margin */
   phosh_drag_surface_set_margin (PHOSH_DRAG_SURFACE (self), margin, 0);
   /* Update drag handle since top-panel size might have changed */
-  update_drag_handle (self, FALSE);
-  /* Trigger redraw and surface commit */
-  gtk_widget_queue_draw (GTK_WIDGET (self));
+  update_drag_handle (self, TRUE);
 
   return FALSE;
 }
@@ -675,18 +724,19 @@ phosh_top_panel_class_init (PhoshTopPanelClass *klass)
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
   PhoshLayerSurfaceClass *layer_surface_class = PHOSH_LAYER_SURFACE_CLASS (klass);
   PhoshDragSurfaceClass *drag_surface_class = PHOSH_DRAG_SURFACE_CLASS (klass);
+  GtkBindingSet *binding_set;
 
   object_class->constructed = phosh_top_panel_constructed;
   object_class->dispose = phosh_top_panel_dispose;
   object_class->set_property = phosh_top_panel_set_property;
   object_class->get_property = phosh_top_panel_get_property;
 
+  widget_class->map = phosh_top_panel_map;
+
   layer_surface_class->configured = phosh_top_panel_configured;
 
   drag_surface_class->dragged = phosh_top_panel_dragged;
 
-  gtk_widget_class_set_css_name (widget_class, "phosh-top-panel");
-
   /**
    * PhoshTopPanel:state:
    *
@@ -717,13 +767,16 @@ phosh_top_panel_class_init (PhoshTopPanelClass *klass)
   g_object_class_install_properties (object_class, PROP_LAST_PROP, props);
 
   signals[ACTIVATED] = g_signal_new ("activated",
-      G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST, 0, NULL, NULL,
-      NULL, G_TYPE_NONE, 0);
+                                     G_TYPE_FROM_CLASS (klass),
+                                     G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                                     0, NULL, NULL, NULL,
+                                     G_TYPE_NONE,
+                                     0);
 
   g_type_ensure (PHOSH_TYPE_ARROW);
 
   gtk_widget_class_set_template_from_resource (widget_class,
-                                               "/sm/puri/phosh/ui/top-panel.ui");
+                                               "/mobi/phosh/ui/top-panel.ui");
   gtk_widget_class_bind_template_child (widget_class, PhoshTopPanel, arrow);
   gtk_widget_class_bind_template_child (widget_class, PhoshTopPanel, batteryinfo);
   gtk_widget_class_bind_template_child (widget_class, PhoshTopPanel, box);
@@ -739,9 +792,15 @@ phosh_top_panel_class_init (PhoshTopPanelClass *klass)
   gtk_widget_class_bind_template_child (widget_class, PhoshTopPanel, menu_system);
   gtk_widget_class_bind_template_child (widget_class, PhoshTopPanel, settings);
   gtk_widget_class_bind_template_child (widget_class, PhoshTopPanel, stack);
+  gtk_widget_class_bind_template_child (widget_class, PhoshTopPanel, top_bar_bin);
   gtk_widget_class_bind_template_callback (widget_class, on_settings_drag_handle_offset_changed);
   gtk_widget_class_bind_template_callback (widget_class, phosh_top_panel_fold);
   gtk_widget_class_bind_template_callback (widget_class, released_cb);
+
+  binding_set = gtk_binding_set_by_class (klass);
+  gtk_binding_entry_add_signal (binding_set, GDK_KEY_Escape, 0, "activated", 0);
+
+  gtk_widget_class_set_css_name (widget_class, "phosh-top-panel");
 }
 
 
@@ -899,3 +958,25 @@ phosh_top_panel_get_state (PhoshTopPanel *self)
 
   return self->state;
 }
+
+
+void
+phosh_top_panel_set_layer (PhoshTopPanel *self, guint32 layer)
+{
+  g_assert (PHOSH_IS_TOP_PANEL (self));
+
+  phosh_layer_surface_set_layer (PHOSH_LAYER_SURFACE (self), layer);
+  phosh_layer_surface_wl_surface_commit (PHOSH_LAYER_SURFACE (self));
+
+  phosh_layer_surface_set_layer (PHOSH_LAYER_SURFACE (self->background), layer);
+  phosh_layer_surface_wl_surface_commit (PHOSH_LAYER_SURFACE (self->background));
+}
+
+
+void
+phosh_top_panel_set_bar_transparent (PhoshTopPanel *self, gboolean transparent)
+{
+  g_return_if_fail (PHOSH_IS_TOP_PANEL (self));
+
+  phosh_util_toggle_style_class (GTK_WIDGET (self->top_bar_bin), "p-solid", !transparent);
+}
